/**
 * @file  AEBuffer.h
 * @brief Schnittstelle fuer die Klasse CAEBuffer.
 */
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_AE3BUFFER_H__594E6CE6_B8A9_4D2C_A283_3AADA6D23606__INCLUDED_)
#define AFX_AE3BUFFER_H__594E6CE6_B8A9_4D2C_A283_3AADA6D23606__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// defines fuer Setup
#include "../Common/AES.h"

// Geraetedateiarchiv einbinden
#include "../Common/DFFileDeviceZip.h"

#pragma pack(1)

// Datensatzgroesse fuer Zutrittslisten (Version 1) definieren.
#define ENTRANCE_RECORD_SIZE_0    22
#define ENTRANCE_RECORD_SIZE_1    35
#define ENTRANCE_RECORD_SIZE_2    19
#define ENTRANCE_RECORD_SIZE_3    16

typedef struct structZeitZone
{
	unsigned char day;
	unsigned short start;
	unsigned short ende;
	unsigned char aktiv;

} STRUCTZeitZone;

typedef struct structZeitModell
{
	unsigned char zeitmodell;
	unsigned short zone;

} STRUCTZeitModell;

typedef struct structPersonalStamm
{
	unsigned char pNr[14];
	unsigned char zeitmodell;
	unsigned char prio;
	
} STRUCTPersonalStamm;

typedef struct structFeiertag
{
	unsigned char zeitmodell;
	unsigned long datum;
	unsigned char prio;

} STRUCTFeiertag;

// Datensatzgrosze fuer Zutrittslisten (Version 2) definieren
#define ENTRANCE2_RECORD_SIZE_0    33
#define ENTRANCE2_RECORD_SIZE_1    64
#define ENTRANCE2_RECORD_SIZE_2    20
#define ENTRANCE2_RECORD_SIZE_3    25
#define ENTRANCE2_RECORD_SIZE_4    21
#define ENTRANCE2_RECORD_SIZE_5    17
#define ENTRANCE2_RECORD_SIZE_6    21

#pragma pack()

class CAEBuffer
{
public:
	CAEBuffer(short channelID);
	virtual ~CAEBuffer();

	void ChangeOrderLong(unsigned char *Buf);
	unsigned long ChangeOrderULong(unsigned long val);
	void ULongToBinMSBFirst(unsigned char *Buf,unsigned long Val);
	void ChangeOrderInt(unsigned char *Buf);

protected:
	unsigned long StartDatensatzB, EndDatensatzB;
	int MaxListB;
	int MaxDatB;

	TDatensatzBeschreibung	DatensatzBeschreibung[MAX_DATENSATZ_BESCHREIBUNG];
	TDatensatzFeld			DatensatzFeld[MAX_DATENSATZ_BESCHREIBUNG][MAX_FELD];

	TDatensatzBeschreibung	ListenBeschreibung[MAX_DATENSATZ_BESCHREIBUNG];
	TDatensatzFeld			ListenFeld[MAX_DATENSATZ_BESCHREIBUNG][MAX_FELD];

	unsigned long m_entrance2Pointer[MAX_DATENSATZ_BESCHREIBUNG];
	STable2                 m_entrance2Table[MAX_DATENSATZ_BESCHREIBUNG];
	STable2Field            m_entrance2TableField[MAX_DATENSATZ_BESCHREIBUNG][MAX_FELD];

public:
	void SetDocumentDefault();
	unsigned long GetMDEEKFlashSize_0400();
	unsigned long GetMDEEKFlashSize();
	unsigned long GetEntranceEKFlashSize();
	unsigned long GetEntrance2EKFlashSize();
	void AdjustExtendedMemPointer();
	void PrepareEntrance2EK(unsigned long offs, unsigned char *buf);
	unsigned long GetFlashSetupSize();


	// Speicher fuer Listendaten
	bool m_listSortCheck[MAX_DATENSATZ_BESCHREIBUNG];
	unsigned long m_listSize[MAX_DATENSATZ_BESCHREIBUNG];
	unsigned char m_ucListRecordSize[MAX_DATENSATZ_BESCHREIBUNG];
	unsigned char *m_pucListMem[MAX_DATENSATZ_BESCHREIBUNG];

	void ClearListBuffer();
	int MakeListe(int ListNum, int ListCnt, int ListSize, unsigned char *Buf);
	
	// Sortieren der Datensaetze ueber ASCII - Feld. Offs ist Offset in Datensatz auf Feldbeginn.
	void QuicksortListRecord(unsigned char ucNum, unsigned char ucOffs, long lLeftRecordNum, long lRightRecordNum);
	void SwapListRecord(unsigned char ucNum, long lLeftRecordNum, long lRightRecordNum);


	// Speicher fuer Zutrittskontrolle (Version 1) - Listendaten
	unsigned long m_ulEntranceListSize[4];
	unsigned long m_ulCompressedListSize[4];
	unsigned char m_ucEntranceRecordSize[4];
	unsigned char *m_pucEntranceListMem[4];
	unsigned char m_pucCompressedListMem[4][FLASH_SIZE_LIST + 1];

	void ClearEntranceListBuffer(int iListNum = -1);
	int MakeEntranceList(int ListNum, int ListCnt, int ListSize, unsigned char *Buf);
	
	// Sortieren der Datensaetze ueber ASCII - Feld. Offs ist Offset in Datensatz auf Feldbeginn.
	void QuicksortEntranceRecord(unsigned char ucNum, unsigned char ucOffs, long lLeftRecordNum, long lRightRecordNum);
	void SwapEntranceRecord(unsigned char ucNum, long lLeftRecordNum, long lRightRecordNum);


	// Speicher fuer Zutrittskontrolle (Version 2) - Listendaten
	bool m_entrance2SortCheck[7];
	unsigned long m_ulEntrance2ListSize[7];
	unsigned char m_ucEntrance2RecordSize[7];
	unsigned char *m_pucEntrance2ListMem[7];

	void ClearEntrance2ListBuffer(int iListNum = -1);

	int CheckEntrance2List(unsigned char num, unsigned short recordCount, unsigned char *buf, unsigned long size);
	int CheckEntrance2Dec(char *buf, int min, int max);
	int CheckEntrance2TM(char *buf);
	int CheckEntrance2Date(char *buf);
	int CheckEntrance2Time(char *buf);
	int CheckEntrance2Weekday(char *buf);
	int MakeEntrance2List(int ListNum, int ListCnt, int ListSize, unsigned char *Buf);
	int ImportEntrance2List(unsigned char num, unsigned short recordCount, unsigned char *buf, unsigned long size);
	
	// Sortieren der Datensaetze ueber ASCII - Feld. Offs ist Offset in Datensatz auf Feldbeginn.
	void QuicksortEntrance2Record(unsigned char ucNum, unsigned char ucOffs, long lLeftRecordNum, long lRightRecordNum);
	void SwapEntrance2Record(unsigned char ucNum, long lLeftRecordNum, long lRightRecordNum);

	// Speicher fuer Timeboy - Listendaten
	unsigned long m_ulTimeboyListSize[MAX_DATENSATZ_BESCHREIBUNG][MAX_DATENSATZ_BESCHREIBUNG];
	unsigned char *m_pucTimeboyListMem[MAX_DATENSATZ_BESCHREIBUNG][MAX_DATENSATZ_BESCHREIBUNG];
	int m_timeboyFlags[MAX_DATENSATZ_BESCHREIBUNG][MAX_DATENSATZ_BESCHREIBUNG];

	void ClearTimeboyListBuffer(unsigned char id = 0xFF, unsigned char num = 0xFF);
	int MakeTimeboyList(unsigned char id, unsigned char num, unsigned short recordCount, unsigned char *buf, unsigned long size, int flags, int *error);

	// fuer Setupdaten
	unsigned char m_pucEepMem[EEP_SIZE];
	unsigned char m_pucFlashMem[FLASH_SIZE_SU + 0x100];
	unsigned char m_pucConfigMem[CONFIG_SIZE_SU + 0x20]; // Mindestens 128Byte
	unsigned char m_pucExtendedMem[EXTENDED_SIZE_SU + 0x100];

	int GetTableCount(unsigned char ucList = FALSE);
	
	int GetTableRecord(unsigned char ucNum,unsigned char *szName, 
		unsigned char *pucFieldCount, unsigned char *pucVariable = NULL, 
		unsigned char ucList = FALSE);

	int GetTableField(unsigned char ucTableNum, unsigned char ucFieldNum, 
		unsigned char *szName, unsigned char *pucType, 
		unsigned char *pucLength, unsigned char ucList = FALSE);
	
	void DefaultIOModule(TIOModule *pIOModule);

	// Dateiarchiv fuer Geraetedaten (Firmware) verwalten
	CDFFileDeviceZip m_fileDevice;

private:
	short m_channelID;

protected:
	void CpyFromKonfigBuf(unsigned char *Des,unsigned char *Src,unsigned long Add,int Cnt);
	int GetTableRecordEntrance(unsigned char ucNum,unsigned char *szName, 
		unsigned char *pucFieldCount);
	int GetTableFieldEntrance(unsigned char ucTableNum, unsigned char ucFieldNum, 
		unsigned char *szName, unsigned char *pucType, 
		unsigned char *pucLength);
	
	// Zutrittskontrolllisten (Version 2) aufsetzen.
	void Entrance2Default();
	int GetTableRecordEntrance2(unsigned char ucNum,unsigned char *szName, 
		unsigned char *pucFieldCount);
	int GetTableFieldEntrance2(unsigned char ucTableNum, unsigned char ucFieldNum, 
		unsigned char *szName, unsigned char *pucType, 
		unsigned char *pucLength);

};

#endif // !defined(AFX_AE3BUFFER_H__594E6CE6_B8A9_4D2C_A283_3AADA6D23606__INCLUDED_)

